﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Dienstfahrten.Logging;

namespace Dienstfahrten.Klassen
{
    public class DienstfahrtListe : IListe
    {
        //Statischer Name der Liste
        public string ListName { get; private set; }
        public SPWeb currentWeb { get; private set; }
        public SPList currentList { get; private set; }
        public List<Object> ObjList { get; private set; }
        Log log;


        //Konstruktor
        public DienstfahrtListe(SPWeb web)
        {
            this.ListName = "Dienstfahrten Liste";
            this.currentWeb = web;
            this.log = new Log();
            this.currentList = currentWeb.Lists[ListName];
        }

        /// <summary>
        /// Liefert alle Einträge aus der Liste
        /// </summary>
        /// <returns>Liste mit allen Einträgen</returns>
        public List<Object> getList()
        {
            List<Object> ResultList = new List<Object>();
            
            foreach (SPListItem item in currentList.Items)
                ResultList.Add(new Dienstfahrt(currentWeb).getById(item.ID));

            return ResultList;
        }

        /// <summary>
        /// Globale Funktionsdefiniton, ersten beiden Parameter müssen Bool sein
        /// </summary>
        /// <param name="pquery">Parameterliste, ersten beiden Parameter müssen bool sein</param>
        /// <returns></returns>
        public List<Object> getListbyParam(params string[] pquery)
        {
            bool favorite = (pquery[0] == "True") ? true : false;
            bool verschickt = (pquery[1] == "True") ? true : false;
            List<string> param = new List<string>();

            for (int i = 2; i < pquery.Length-1; i++)
			{
			 param.Add(pquery[i]);
			}

            return getListbyParam(favorite, verschickt, param.ToArray());
        }

        /// <summary>
        /// Liefert Einträge aus der Liste anhand der übergebenen Parameter
        /// </summary>
        /// <param name="query">Filterparameter</param>
        /// <returns>Liste mit Einträgen</returns>
        public List<Object> getListbyParam(bool favorit, bool verschickt, params string[] pquery)
        {
            //Baue where Klausel zusammen
            string concatQuery = string.Empty;
            foreach (string q in pquery)
            {
                concatQuery += q;
            }


            //Erstelle Abfrage
            SPQuery query = new SPQuery();
            if(concatQuery!= string.Empty)
            query.Query = string.Format(@"
                                            <Where>
                                                <And>  
                                                    <Eq>
				                                        <FieldRef Name='Favorit' /><Value Type='Boolean'>{0}</Value>
			                                        </Eq>
                                                    <And>
			                                            <Eq>
				                                            <FieldRef Name='Verschickt' /><Value Type='Boolean'>{1}</Value>
			                                            </Eq>
                                                        <And>
		                                                    <Eq>
			                                                    <FieldRef Name='Author' /><Value Type='User'>{2}</Value>
		                                                    </Eq>
                                                            {3}
                                                        </And>
                                                    </And>
                                                </And>
                                            </Where>",
                                        (favorit) ? "1" : "0",
                                        (verschickt) ? "1" : "0",
                                        currentWeb.CurrentUser.Name,
                                        concatQuery);
            else
            //Eine <AND> </And> Klammer muss immer genau 2 Elemente umschließen
            query.Query = string.Format(@"
                                            <Where>
                                                <And>  
                                                    <Eq>
				                                        <FieldRef Name='Favorit' /><Value Type='Boolean'>{0}</Value>
			                                        </Eq>
                                                    <And>
			                                            <Eq>
				                                            <FieldRef Name='Verschickt' /><Value Type='Boolean'>{1}</Value>
			                                            </Eq>
		                                                <Eq>
			                                                <FieldRef Name='Author' /><Value Type='User'>{2}</Value>
		                                                </Eq>
                                                    </And>
                                                </And>
                                            </Where>", 
                                        (favorit) ? "1" : "0",
                                        (verschickt) ? "1" : "0",
                                        currentWeb.CurrentUser.Name
                                        );

            List<Object> ResultList = new List<Object>();
            SPSecurity.RunWithElevatedPrivileges(
                delegate()
                {
                    var log = new Log();

                    using (var site = new SPSite(currentWeb.Site.ID))
                    {
                        using (SPWeb web = site.OpenWeb(currentWeb.ID))
                        {
                            SPListItemCollection ItemColl = null;
                            SPList curlist = web.Lists[ListName];
                            ItemColl = curlist.GetItems(query);

                            //Erstelle Liste mit Objekten
                            if (ItemColl != null && ItemColl.Count > 0)
                                foreach (SPListItem item in ItemColl)
                                    ResultList.Add(new Dienstfahrt(currentWeb).getById(item.ID));

                        }
                    }
                }
             );
            return ResultList;

        }

        public List<Object> getListByDatum(DateTime start, DateTime ende, bool favorit, bool verschickt, string pquery)
        {
            string query = string.Format(@"
                                        <And>
                                            <Geq>
                                                <FieldRef Name='Datum' />
                                                <Value Type='DateTime'>{0}</Value>
                                            </Geq>
                                            <Leq>
                                                <FieldRef Name='Datum' />
                                                <Value Type='DateTime'>{1}</Value>
                                            </Leq>
                                        </And>",
                                        start.ToString("yyyy-MM-dd"),
                                        ende.ToString("yyyy-MM-dd"));

            return getListbyParam(favorit, verschickt, query, pquery);
        }


        public List<Object> getListByQuartal(int quartal1, int quartal2, int jahr, bool favorit, bool verschickt, string pquery)
        {
            DateTime Qstart = this.QuarterStart(quartal1, jahr);
            DateTime Qend = this.QuarterEnd(quartal2, jahr);
            return getListByDatum(Qstart, Qend, favorit, verschickt, pquery);
        }


        //#################
        //Monats-Abfragen
        //#################        
        public List<Object> getListbyMonat(int monat, int jahr, bool favorit, bool verschickt, string pquery)
        {
            DateTime start = new DateTime(jahr, monat, 1);
            DateTime ende =  start.AddMonths(1).AddDays(-1);

            return getListByDatum(start, ende, favorit, verschickt, pquery);
        }

        //#################
        //Quartals-Abfragen
        //#################        
        public List<Object> getListbyAktuellesQuartal(bool favorit, bool verschickt, string pquery)
        {
            int aktuellesQuartal = (DateTime.Now.Month - 1) / 3 + 1;

            return getListByQuartal(aktuellesQuartal, aktuellesQuartal, DateTime.Now.Year, favorit, verschickt, pquery);
        }

        public List<Object> getListbyQuartal1(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(1, 1, DateTime.Now.Year, favorit, verschickt, pquery);
        }

        public List<Object> getListbyQuartal2(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(2, 2, DateTime.Now.Year, favorit, verschickt, pquery);
        }

        public List<Object> getListbyQuartal3(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(3, 3, DateTime.Now.Year, favorit, verschickt, pquery);
        }

        public List<Object> getListbyQuartal4(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(4, 4, DateTime.Now.Year, favorit, verschickt, pquery);
        }

        public List<Object> getListbyQuartal4Vorjahr(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(4, 4, DateTime.Now.Year-1, favorit, verschickt, pquery);
        }

        public List<Object> getListbyQuartal1Folgejahr(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(1, 1, DateTime.Now.Year + 1, favorit, verschickt, pquery);
        }

        //###################
        //Halbjahres-Abfragen
        //###################        
        public List<Object> getListbyHalbjahr1(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(1, 2, DateTime.Now.Year, favorit, verschickt, pquery);
        }

        public List<Object> getListbyHalbjahr2(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(1, 3, DateTime.Now.Year, favorit, verschickt, pquery);
        }

        //###############
        //Jahres-Abfragen
        //###############        
        public List<Object> getListbyAktuellesJahr(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(1, 4, DateTime.Now.Year, favorit, verschickt, pquery);
        }

        public List<Object> getListbyFolgejahr(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(1, 4, DateTime.Now.Year + 1, favorit, verschickt, pquery);
        }

        public List<Object> getListbyVorjahr(bool favorit, bool verschickt, string pquery)
        {
            return getListByQuartal(1, 4, DateTime.Now.Year - 1, favorit, verschickt, pquery);
        }



        public List<Object> getFavorites(string pquery)
        {
            return getListbyParam(true, false, pquery);
        }
        



        //###################
        //Helper
        //###################
   /// <summary>
   /// Konstruktor. Berechnet das Start- und das
   /// Enddatum des übergebenen Quartals.
   /// </summary>
   /// <param name="quarter">Das Quartal</param>
   /// <param name="year">Das Jahr</param>
   /// http://dotnet-snippets.de/dns/den-ersten-und-den-letzten-tag-eines-quartals-berechnen-SID259.aspx
   public DateTime QuarterStart(int quarter, int year)
   {
      // Das übergebene Quartal und das Jahr überprüfen
      if (quarter >= 1 && quarter <= 4 &&
         year >= 0 && year <= 9999)
      {

         int month = (quarter * 3) - 2;
         // Ersten Tag im Quartal berechnen
         return new DateTime(year, month, 1);

      }
      else
      {
         throw new ArgumentException("Das Quartal muss eine Zahl " +
            "zwischen 1 und 4 und das Jahr eine Zahl zwischen 0 " +
            "und 9999 sein");
      }
   }

   public DateTime QuarterEnd(int quarter, int year)
   {
      // Das übergebene Quartal und das Jahr überprüfen
      if (quarter >= 1 && quarter <= 4 &&
         year >= 0 && year <= 9999)
      {
         int month = (quarter * 3) - 2;
         // Ersten Tag im Quartal berechnen
         DateTime startDate = new DateTime(year, month, 1);

         // Letzten Tag im Quartal berechnen
         return startDate.AddMonths(3).AddDays(-1);
      }
      else
      {
         throw new ArgumentException("Das Quartal muss eine Zahl " +
            "zwischen 1 und 4 und das Jahr eine Zahl zwischen 0 " +
            "und 9999 sein");
      }
   }

    }
}
